/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, NX*/

/**
 * Licensing controller.
 *
 * @since 3.0
 */
Ext.define('NX.proui.controller.Licensing', {
  extend: 'NX.app.Controller',
  requires: [
    'NX.Conditions',
    'NX.Permissions',
    'NX.Messages',
    'NX.Security',
    'NX.I18n'
  ],

  views: [
    'licensing.LicenseAgreement',
    'licensing.LicensingDetails'
  ],

  refs: [
    {
      ref: 'panel',
      selector: 'nx-proui-licensing-details'
    }
  ],

  /**
   * @override
   */
  init: function () {
    var me = this;

    me.getApplication().getFeaturesController().registerFeature({
      mode: 'admin',
      path: '/System/Licensing',
      text: NX.I18n.get('Licensing_Text'),
      description: NX.I18n.get('Licensing_Description'),
      view: { xtype: 'nx-proui-licensing-details' },
      iconConfig: {
        file: 'license_key.png',
        variants: ['x16', 'x32']
      },
      visible: function () {
        return NX.Permissions.check('nexus:licensing:read');
      }
    }, me);

    me.listen({
      component: {
        'nx-proui-licensing-details button[action=install]': {
          click: me.install,
          afterrender: me.bindInstallButton
        },
        'nx-proui-licensing-details button[action=uninstall]': {
          click: me.uninstall,
          afterrender: me.bindUninstallButton
        },
        'nx-proui-licensing-agreement button[action=agree]': {
          click: me.onAgree
        }
      }
    });
  },

  /**
   * @private
   */
  showAgreementWindow: function () {
    return Ext.widget('nx-proui-licensing-agreement');
  },

  /**
   * @private
   * Install license, as user agreed.
   */
  onAgree: function (button) {
    var win = button.up('window'),
        form = win.licenseForm,
        authToken = win.authToken;

    win.close();
    form.submit({
      params: {
        authToken: authToken
      },
      success: function () {
        form.isValid();
        NX.Messages.add({ text: NX.I18n.get('Licensing_Install_Success'), type: 'success' });
        this.getController('Refresh').refresh();
      }
    });
  },

  /**
   * @override
   * @private
   * Enable 'Install' when user has 'create' permission.
   */
  bindInstallButton: function (button) {
    button.mon(
        NX.Conditions.and(
            NX.Conditions.isPermitted('nexus:licensing:create')
        ),
        {
          satisfied: button.enable,
          unsatisfied: button.disable,
          scope: button
        }
    );
  },

  /**
   * @override
   * @private
   * Enable 'Uninstall' when user has 'delete' permission.
   */
  bindUninstallButton: function (button) {
    button.mon(
        NX.Conditions.and(
            NX.Conditions.isPermitted('nexus:licensing:delete')
        ),
        {
          satisfied: button.enable,
          unsatisfied: button.disable,
          scope: button
        }
    );
  },

  /**
   * @private
   * Install a license after successful authentication.
   */
  install: function (button) {
    var me = this,
        form = button.up('form');
    NX.Security.doWithAuthenticationToken(
        NX.I18n.format('Licensing_Authentication_Validation', 'Installing'),
        {
          success: function (authToken) {
            Ext.apply(me.showAgreementWindow(), { licenseForm: form, authToken: authToken });
          }
        }
    );
  },

  /**
   * @private
   * Uninstall current license after successful authentication.
   */
  uninstall: function () {
    NX.Security.doWithAuthenticationToken(
        NX.I18n.format('Licensing_Authentication_Validation', 'Uninstalling'),
        {
          success: function (authToken) {
            NX.direct.licensing_Licensing.uninstall(authToken, function (response) {
              if (Ext.isDefined(response) && response.success) {
                NX.Messages.add({ text: NX.I18n.get('Licensing_Uninstall_Success'), type: 'success' });
              }
            });
          }
        }
    );
  }

});
